#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <time.h>

#define PATCHCOUNT 6
#define PATCHSIZE 0x10

const char Patches[PATCHCOUNT][PATCHSIZE] = {
	{0x69, 0x77, 0x64, 0x32, 0x66, 0x69, 0x78, 0x65, 0x73, 0x2E, 0x64, 0x6C, 0x6C, 0x00, 0x00, 0x00 },
	{0xE9, 0xE2, 0x64, 0x40, 0x00, 0x90, 0x90, 0x90, 0x90, 0x90, 0xFF, 0x15, 0xB8, 0x72, 0x84, 0x00 },
	{0xFA, 0xFF, 0x68, 0xF0, 0x0F, 0x40, 0x00, 0xEB, 0x1C, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC },
	{0xE9, 0x63, 0x28, 0xFA, 0xFF, 0xFF, 0x15, 0xB8, 0x72, 0x84, 0x00, 0xEB, 0x15, 0xCC, 0xCC, 0xCC },
	{0xFA, 0xFF, 0xC7, 0x86, 0x9E, 0x06, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xEB, 0x1C, 0xCC, 0xCC },
	{0xB8, 0x98, 0x00, 0x8A, 0x00, 0xE9, 0x1E, 0x28, 0xFA, 0xFF, 0xE9, 0xB6, 0x9A, 0xBF, 0xFF, 0xCC }
};

const uint32_t PatchLocations[PATCHCOUNT] = {0xFF0, 0x3E82B, 0x444D10, 0x444D30, 0x444D50, 0x444D70 };
const char PatchPatterns[PATCHCOUNT][PATCHSIZE] = {
	{0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
	{0xC7, 0x86, 0x9E, 0x06, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0xFF, 0x15, 0xB8, 0x72, 0x84, 0x00},
	{0xFA, 0xFF, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC},
	{0xE9, 0x63, 0x28, 0xFA, 0xFF, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC},
	{0xFA, 0xFF, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC},
	{0xB8, 0x98, 0x00, 0x8A, 0x00, 0xE9, 0x1E, 0x28, 0xFA, 0xFF, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC}
	};

unsigned char FileVersionString[28] = {
	0x01, 0x00, 0x46, 0x00, 0x69, 0x00, 0x6C, 0x00, 0x65, 0x00, 0x56, 0x00,
	0x65, 0x00, 0x72, 0x00, 0x73, 0x00, 0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00,
	0x00, 0x00, 0x00, 0x00
};

const char ProductVersionString[32] = {
	0x01, 0x00, 0x50, 0x00, 0x72, 0x00, 0x6F, 0x00, 0x64, 0x00, 0x75, 0x00,
	0x63, 0x00, 0x74, 0x00, 0x56, 0x00, 0x65, 0x00, 0x72, 0x00, 0x73, 0x00,
	0x69, 0x00, 0x6F, 0x00, 0x6E, 0x00, 0x00, 0x00
};

const char versionString[20] = {
	0x43, 0x00, 0x75, 0x00, 0x73, 0x00, 0x74, 0x00, 0x6F, 0x00, 0x6D, 0x00,
	0x44, 0x00, 0x4C, 0x00, 0x4C, 0x00, 0x00, 0x00
};
void waitForInput() {
	printf("\nPress Any Key to Continue\n");  
	getch();  
}
int main(int argc, char** argv) {
	char* fname = calloc(1,1024);
	if (argc <2) {
		strcpy(fname,"iwd2.exe");
	}else{
		strcpy(fname,argv[1]);
	}
	printf("Loading file: %s\n",fname);
	FILE* f = fopen(fname,"rb");
	if (f==NULL) {
		printf("Could not open file: %s\n",fname);
		waitForInput();
		return -1;
	}else{
		//verify patch not applied and file can be patched
		char line[PATCHSIZE] = {0};
		uint32_t i=0;
		for (i=0;i<PATCHCOUNT;i++) {
			printf("Checking location for Patch %d\n",i+1);
			fseek(f,PatchLocations[i],0);
			memset(line,0,PATCHSIZE);
			fread(line,PATCHSIZE,1,f);
			if (memcmp(line,PatchPatterns[i],PATCHSIZE)!=0) {
				if (memcmp(line,Patches[i],PATCHSIZE)==0) {
					printf("Found patch %d already applied.\nFile not patchable. File may already be patched.\n",i+1);
				}else{
					printf("File not patchable, cannot find patch pattern %d at location %x\n",i+1,PatchLocations[i]);
				}
				fclose(f);
				return;
			}
		}
	}
	fclose(f);

	char nfname[1024] = {0};
	time_t curtime;
	struct tm * ctm;
	time(&curtime);
	ctm = localtime(&curtime);
	sprintf_s(nfname,1024,"%s.%d.bak",fname,curtime);
	FILE* cf=fopen(nfname,"wb");
	if (cf==NULL) {
		printf("Unable to backup %s to %s\n",fname,nfname);
		waitForInput();
		return -1;
	}

	f=fopen(fname,"rb+");
	char c;
	printf("backing up %s\n",fname);
	while ((c=getc(f))!=EOF || feof(f)==0 ) {
		fputc(c,cf);
	}
	printf("backup complete\n");
	fclose(cf);
	uint32_t i=0;
	for (i=0;i<PATCHCOUNT;i++) {
		printf("Applying patch %d\n",i+1);
		fseek(f,PatchLocations[i],0);
		fwrite(Patches[i],PATCHSIZE,1,f);
	}
	i=48;
	fseek(f,-32,SEEK_END);
	//printf("%x\t",ftell(f));
	char FileInfoLocator[32]=  {0};
	printf("Modifying File Version\n");
	int changeCount=0;
	while (1) {
		memset(FileInfoLocator,0,32);
	    int readCount = fread(FileInfoLocator,32,1,f);
		if (feof(f) == 0 && ferror(f)==0) {
			if (memcmp(FileInfoLocator,FileVersionString,28)==0) {
				fseek(f,-(i-28),SEEK_END);
				fwrite(versionString,20,1,f);
				fseek(f,-i,SEEK_END);
				changeCount++;
			}
			if (memcmp(FileInfoLocator,ProductVersionString,32)==0) {
				fseek(f,-(i-32),SEEK_END);
				fwrite(versionString,20,1,f);
				fseek(f,-i,SEEK_END);
				changeCount++;
			}
		}else{
			printf("Could not modify file version\n");
			//printf("%x\t",ftell(f));
			break;
		}
		if (i >= 0xA000 || changeCount > 1) {
			break;
		}
		i++;
		fseek(f,-i,SEEK_END);
		
		//printf("%x\t",ftell(f));
	}
	fclose(f);
	printf("done\n");
	waitForInput();
	return 0;
}